function [payoff1,payoff2]=calc_pairwise_payoff(s1,s2,b,c,E, nRuns,states,gameLengths)
% calc_pairwise_payoff simulates play between two strategies s1 and s2 over 
% many games, and returns the average payoff of each.
%
% Inputs: 
%
% s1 = strategy 1 (specified as per definitions below)
% s2 = strategy 1 (specified as per definitions below)
% b = benefit of cooperation
% c = cost of cooperation
% E = error rate
% nRuns = number of games to simulate
% states = a matrix of each possible memory2 state (passed instead of
%          generated so that you dont have to generate it for each pair).
%          To generate 'states' use the following code:
%             states=[];
%             for st1=[1 0]
%                 for st2=[1 0]
%                     for st3=[1 0]
%                         for st4=[1 0]
%                             states=[states; st1 st2 st3 st4];
% 
%                         end
%                     end
%                 end
%             end
% gameLengths = an nRuns x 1 vector containing the length of each game
%               (passed instead of generated so that all pairwise payoff
%               simulations can use the same set of game lengths, removing
%               a source of random variation).
%               To generate 'gameLengths' for continuation probability d:
%                 gameLengths=1+geornd(1-d,nRuns,1)
%
% Outputs:
% 
% payoff1 = average per-game payoff of strategy s1
% payoff2 = average per-game payoff of strategy s2



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STRATEGY SPECIFICATION FORMAT
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Strategies can have several forms:
% a) a 16 digit vector, specifying a memory-2 strategy - ie giving the
%    strategy's move (0=D, 1=C) in each of the 16 possible states of the form
%    (your move in t-2, others move in t-2, your move in t-1, others move in t-1)
%    in the order CCCC,CCCD,CCDC,CCDD...
% b) a 2 digit vector, first digit=0.5, 2nd digit specifying # of successive Ds to trigger grim
%    (i.e. Grim = [0 1], Grim2 = [0 2], Grim3 = [0 3])
% c) a 2 digit vector, first digit=1, 2nd digit specifying # of Ds to trigger defection by a TFxT strategy
%    (i.e. TF3T = [1 3])
% d) a 2 digit vector, first digit=2, 2nd digit specifying # of rounds to play C before switching to D
%    (i.e. C-to-ALLD = [2 1])
% e) one of various directly coded strategies: 
%    [4 1]=D-TFT
%    [4 2]=D-TF2T
%    [4 2.5]=D-TF3T
%    [4 3]=DC-Alt
%    [4 6]=D-Grim2
%    [4 7]=D-Grim3
%    [5 1]=2TF2T
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

rand('state',floor(sum(clock*10000)));


% initialize variables keeping track of whether player 1 has a hardcoded
% strategy
p1GrimX=0;
p1TFXT=0;
p1Mach=0;
p1DTFT=0;
p1DTF2T=0;
p1DTF3T=0;
p1DG2X=0;
p1DG3X=0;
p1Alt=0;
p12TF2T=0;
if length(s1)>2
elseif length(s1)==2
    if s1(1)==0.5
        p1GrimX=1;
    elseif s1(1)==1
        p1TFXT=1;
    elseif s1(1)==2
        p1Mach=s1(2);        
    elseif s1(1)==4 & s1(2)==1
        p1DTFT=1;
    elseif s1(1)==4 & s1(2)==2
        p1DTF2T=1;
    elseif s1(1)==4 & s1(2)==2.5
        p1DTF3T=1;
    elseif s1(1)==4 & s1(2)==3
        p1Alt=1;
    elseif s1(1)==4 & s1(2)==6
        p1DG2X=1;
    elseif s1(1)==4 & s1(2)==7
        p1DG3X=1;
    elseif s1(1)==5 & s1(2)==1
        p12TF2T=1;
    end
end



% initialize variables keeping track of whether player 1 has a hardcoded
% strategy
p2GrimX=0;
p2TFXT=0;
p2Mach=0;
p2DTFT=0;
p2DTF2T=0;
p2DTF3T=0;
p2DG2X=0;
p2DG3X=0;
p2Alt=0;
p22TF2T=0;
if length(s2)>2
elseif length(s2)==2
    if s2(1)==0.5
        p2GrimX=1;
    elseif s2(1)==1
        p2TFXT=1;
    elseif s2(1)==2
        p2Mach=s2(2);        
    elseif s2(1)==4 & s2(2)==1
        p2DTFT=1;
    elseif s2(1)==4 & s2(2)==2
        p2DTF2T=1;
    elseif s2(1)==4 & s2(2)==2.5
        p2DTF3T=1;
    elseif s2(1)==4 & s2(2)==3
        p2Alt=1;
    elseif s2(1)==4 & s2(2)==6
        p2DG2X=1;
    elseif s2(1)==4 & s2(2)==7
        p2DG3X=1;
    elseif s2(1)==5 & s2(2)==1
        p22TF2T=1;
    end
end



% initialize payoffs
payoffs=[0 0];

% step through each of nRun games
for i=1:nRuns
       

    % the variable 'state' keep track of the last 5 moves by each player,
    % in the format [p1(t-5) p2(t-5) p1(t-4) p2(t-4) ... p2(t-1)], with 0=D
    % and 1=C. We begin by initializing things as if everyone had played C
    state=ones(1,10);

    % with store of history of each game
    gameHere=[];

    
    % keeps track of if Grim strategies have triggered
    G_last1_trig=0;
    G_last2_trig=0;
    G_last3_trig=0;
      
    % step through each period of the current game
    for period=1:gameLengths(i)

       
        % for C-to-ALLD, assign what it is really doing this period
        if p1Mach>0
            % Mach
            if period<=p1Mach
                s1=1*ones(1,16); % ALLC
            else
                s1=0*ones(1,16); % ALLD
            end
        end
        if p2Mach>0
            % Mach
            if period<=p2Mach
                s2=1*ones(1,16); % ALLC
            else
                s2=0*ones(1,16); % ALLD
            end
        end
        
        
        % for the D-first strategies, assign what they are really doing
        % this round
        if p1DTFT==1
            if period==1
                s1=0*ones(1,16);
            else
                s1=[1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0];
            end
        elseif p1DTF2T==1
            if period==1
                s1=0*ones(1,16);
            else
                s1=[1 1 1 1 1 0 1 0 1 1 1 1 1 0 1 0];
            end
        elseif p1DTF3T==1
            if period==1
                s1=0*ones(1,16);
                p1TFXT=0;
            else
                s1=[1 3];
                p1TFXT=1;
            end
        elseif p1DG2X==1
            if period==1
                s1=0*ones(1,16);
                p1GrimX=0;
            else
                s1=[0.5 2];
                p1GrimX=1;
            end
        elseif p1DG3X==1
            if period==1
                s1=0*ones(1,16);
                p1GrimX=0;
            else
                s1=[0.5 3];
                p1GrimX=1;
            end
        end        
        if p2DTFT==1
            if period==1
                s2=0*ones(1,16);
            else
                s2=[1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0];
            end
        elseif p2DTF2T==1
            if period==1
                s2=0*ones(1,16);
            else
                s2=[1 1 1 1 1 0 1 0 1 1 1 1 1 0 1 0];
            end
        elseif p2DTF3T==1
            if period==1
                s2=0*ones(1,16);
                p2TFXT=0;
            else
                s2=[1 3];
                p2TFXT=1;
            end
        elseif p2DG2X==1
            if period==1
                s2=0*ones(1,16);
                p2GrimX=0;
            else
                s2=[0.5 2];
                p2GrimX=1;
            end
        elseif p2DG3X==1
            if period==1
                s2=0*ones(1,16);
                p2GrimX=0;
            else
                s2=[0.5 3];
                p2GrimX=1;
            end
        end
        
        
        % adjust GrimX triggers
        if ((state(9)==0 | state(10)==0))
            % if Ds in the last period, trigger Grim
            G_last1_trig=1;
        end
        if ((state(9)==0 | state(10)==0) & (state(7)==0 | state(8)==0))
            % if Ds in the last 2 periods, trigger Grim2
            G_last2_trig=1;
        end
        if ((state(9)==0 | state(10)==0) & (state(7)==0 | state(8)==0)& (state(5)==0 | state(6)==0))
            % if Ds in the last 3 periods, trigger Grim3
            G_last3_trig=1;
        end
        
        
        % what did P1 mean to do?
        if p1TFXT==0 & p1Alt==0 & p1GrimX==0 & p12TF2T==0 
            % if P1 is not any of these hardcoded strategies, look up P1's
            % move based on his strategy
            p1desire=s1(find(state(7)==states(:,1) & state(8)==states(:,2) & state(9)==states(:,3) & state(10)==states(:,4)));
        elseif p1GrimX==1
            % if its a Grim strat, then it plays C as long as its not
            % triggered
                if s1(2)==1
                    p1desire=1-G_last1_trig;
                elseif s1(2)==2
                    p1desire=1-G_last2_trig;
                elseif s1(2)==3
                    p1desire=1-G_last3_trig;
                end
        elseif p1TFXT==1
            % if its a TFXT strat, look at p2's last X moves - where they all D?
            p1desire=1-((sum(state(10:-2:(10-(s1(2)-1)*2))==0)==length(state(10:-2:(10-(s1(2)-1)*2)))));
        elseif p1Alt==1
            % if its DC-Alt...
            p1desire=1-mod(period,2);
        elseif p12TF2T==1
            % if its 2TF2T...
            if (state(10)==0 & state(8)==0 & state(6)==1) ...
                    | (state(10)==1 & state(8)==0 & state(6)==0) ...
                    | (state(10)==0 & state(8)==0 & state(6)==0)
                p1desire=0;
            else
                p1desire=1;
            end
        end
        

        % what did P2 mean to do
        if p2TFXT==0 & p2Alt==0 & p2GrimX==0 & p22TF2T==0 
            % if P2 is not any of these hardcoded strategies, look up P1's
            % move based on his strategy, keeping in mind that things looked flipped to him 
            p2desire=s2(find(state(8)==states(:,1) & state(7)==states(:,2) & state(10)==states(:,3) & state(9)==states(:,4)));
        elseif p2GrimX==1
            % if its a Grim strat, then it plays C as long as it hasnt
            % triggered
                if s2(2)==1
                    p2desire=1-G_last1_trig;
                elseif s2(2)==2
                    p2desire=1-G_last2_trig;
                elseif s2(2)==3
                    p2desire=1-G_last3_trig;;
                end
        elseif p2TFXT==1
            % if its a TFXT strat, look at p2's last X moves - where they
            % all D?
            p2desire=1-(sum(state(9:-2:(9-(s2(2)-1)*2))==0)==length(state(9:-2:(9-(s2(2)-1)*2))));
        elseif p2Alt==1
            % DC-Alt...
            p2desire=1-mod(period,2);
        elseif p22TF2T==1
            % 2TF2T...
            if (state(9)==0 & state(7)==0 & state(5)==1) ...
                    | (state(9)==1 & state(7)==0 & state(5)==0) ...
                    | (state(9)==0 & state(7)==0 & state(5)==0)
                p2desire=0;
            else
                p2desire=1;
            end
        end

        % determine whether each strategy will make an error this period
        p1E=rand<E;
        p2E=rand<E;
           
        % if no error, then move = desire; if error, then move = 1-desire
        p1move=p1desire*(1-p1E)+(1-p1desire)*p1E;
        p2move=p2desire*(1-p2E)+(1-p2desire)*p2E;
        
        % append the current moves onto the end of 'state', and throw out
        % the moves from 5 periods ago
        state=[state([3:10]) p1move p2move];


        % update payoffs based on the actual moves
        payoffs=payoffs+[-c*p1move+b*p2move -c*p2move+b*p1move];
        
    end % period
end % i (game)

% average payoff of the number of games
payoffs=payoffs/nRuns;


payoff1=payoffs(1);
payoff2=payoffs(2);