function pis=calc_pis(posStrats,startFrom,endAt,data,bc,error,numSubjects)
% calc_pis returns a [i x s] matrix of probabilities, corresponding to the 
% likelihood that each of i subjects with histories in 'data' uses each 
% of the s strategies listed in 'posStrats'.
%
% INPUTS:
% posStrats = list of strategies to explore (using the key given below)
% startFrom = first interaction in 'data' to consider
% endAt = last ineraction in 'data' to consider
% data = data file listing all decisions
% bc = the b/c ratio for the games listed in 'data'
% error = error rate for games listed in 'data'
% numSubjects = number of subjects in 'data
%
% OUTPUT
% pis = i x s matrix of pi(s) values (likelihood of subject i's history if
%       she was playing strategy s)
% 
%%%%%%%%%%%%%%%%%%%%%%%
% KEY FOR 'posStrats' 
%%%%%%%%
% * to define an arbitrary strategy that conditions on the previous round 
% plus the other's move 2 rounds ago, give an 8 unit vector giving
% response (-1=D, 1=C) to each of the 8 possible outcomes of 
% (other t-2, you t-1, other t-1), in the following order: 
% [CCC, CCD, CDC, CDD, DCC, DCD, DDC, DDD]
% so for example:
% ALLC = [1 1 1 1 1 1 1 1]
% ALLD = [-1 -1 -1 -1 -1 -1 -1 -1]
% TFT = [1 -1 1 -1 1 -1 1 -1]
% TF2T = [1 1 1 1 1 -1 1 -1]
% 2TFT = [1 -1 1 -1 -1 -1 -1 -1]
% PTFT = [1 -1 -1 1 1 -1 -1 1]
%%%%%%%
% * strategies of interest which dont have this form are hardcoded using the following flags:
% -2.2*ones(1,8) = 2TF2T
% -3*ones(1,8) = TF3T
% -6*ones(1,8) = D-TFT 
% -7*ones(1,8) = D-TF2T
% -7.1*ones(1,8) = D-TF3T
% -7.15*ones(1,8) = D-Grim
% -7.2*ones(1,8) = D-Grim2
% -7.3*ones(1,8) = D-Grim3
% -8*ones(1,8) = DC-Alt
% -9*ones(1,8) = Grim
% -10*ones(1,8) = C-to-ALLD
% -18*ones(1,8) = Grim-2
% -27*ones(1,8) = Grim-3
% -30*ones(1,8) = 2PTFT
% -99*ones(1,8) = T2 (as implemented by Dal Bo & Frechette 2011)
%%%%%%%%%%%%%%%%%%%%%%%%%


syms G 

% will hold likelihood for subject i of strategy s - here we initialize it
% to be all ones (and multiple and divide by G so that matlab considers it
% a matrix of symbolic values)
pis=G*ones(numSubjects,length(posStrats(:,1))+2)/G;

% loop through each subject
for s=1:numSubjects
    [s numSubjects]

    % loop through each interaction, starting from startFrom and ending at endAt
    for m=startFrom:endAt
            
          
            % a dummy variable to keep track of the
            % state of the T2 trigger.
            % 0 = happy
            % 1 = 1st punishment is this round
            % 2 = 2nd punishment is this round 
            T2trigger=0;
            
            % dummies to keep track of whether each Grim strategy has
            % triggered
            G_last2_trig=0;
            G_last3_trig=0;
            
            
            % loop through each period of interacion m 
            for p=1:max(data(find((data(:,1)==m & data(:,3)==s)),2))
                
                %%%%%%%%%%%%%%%%%%%%
                % ESTABLISH HISTORY BASED ON DATA
                %%%%%%%%%%%%%%%%%%%%
                % 'data' has one row per decision, with the following
                % relevant columns:
                % 1 = interaction
                % 2 = period
                % 3 = subject
                % 13 = actual move (1=C, 2=D)
                % 18 = intended move (1=C, 2=D)
                % 19 = other player's actual move (1=C, 2=D)
                %%%%%%%%%%%%%%%%
                
                
                % MOVES LAST ROUND
                % if p=1, assume everyone played C last move
                if p==1
                    you_p_1=1;
                    other_p_1=1;
                else
                    % your move in p-1 (1=C,2=D)
                    you_p_1=length(find((data(:,1)==m & data(:,2)==(p-1) & data(:,3)==s & data(:,13)==2)))+1;
                    % other's move in p-1 (1=C,2=D)
                    other_p_1=length(find((data(:,1)==m & data(:,2)==(p-1) & data(:,3)==s & data(:,19)==2)))+1;
                end

                % MOVES 2 ROUNDS AGO
                % if p=1 or 2, assume evryone placed C 2 moves ago
                if p<3
                    you_p_2=1;
                    other_p_2=1;
                else
                    % your move in p-2 (1=C,2=D)
                    you_p_2=length(find((data(:,1)==m & data(:,2)==(p-2) & data(:,3)==s & data(:,13)==2)))+1;
                    % other's move in p-2 (1=C,2=D)
                    other_p_2=length(find((data(:,1)==m & data(:,2)==(p-2) & data(:,3)==s & data(:,19)==2)))+1;
                end
                
                % others move 3 rounds ago
                if p<4
                    you_p_3=1;
                    other_p_3=1;
                else
                    you_p_3=length(find((data(:,1)==m & data(:,2)==(p-3) & data(:,3)==s & data(:,13)==2)))+1;
                    other_p_3=length(find((data(:,1)==m & data(:,2)==(p-3) & data(:,3)==s & data(:,19)==2)))+1;
                end
               

               
                % UPDATE T2 trigger
                % if either player defected, and T2trigger
                % is 0, then start the T2trigger
                if (you_p_1==2 || other_p_1==2) && T2trigger==0
                    T2trigger=1;
                    % if both people played C, no
                    % problem - leave T2trigger alone
                elseif you_p_1==1 && other_p_1==1 && T2trigger==0
                    T2trigger=0;
                    % otherwise, increment T2trigger
                elseif T2trigger==1 
                    T2trigger=2;
                elseif T2trigger==2
                    T2trigger=0;
                end

                
                % UPDATE Grim triggers
                % for Grim, you just need to check if there has ever been a
                % D
                been_1_D=((length(find(data(:,1)==m & data(:,2)<p & data(:,3)==s & (data(:,13)==2 | data(:,19)==2))))>0);
                % if either player played D in the last 2 rounds, Grim2 is triggered
                if (you_p_1==2 || other_p_1==2) && (you_p_2==2 || other_p_2==2)
                    G_last2_trig=1;
                end
                % if either player played D in the last 3 rounds, Grim3 is triggered
                if (you_p_1==2 || other_p_1==2) && (you_p_2==2 || other_p_2==2) && (you_p_3==2 || other_p_3==2)
                    G_last3_trig=1;
                end
                
                


                
                
                % RECORD THE SUBJECTS REAL [INTENDED] CHOICE THIS ROUND (0=D,1=C)
                y_imr=length(find((data(:,1)==m & data(:,2)==p & data(:,3)==s & data(:,18)==1)));


                

                % now step through each strategy in posStrats, and figue
                % out what it's predicted move s_imr is given the current
                % history (-1=D, 1=C)
                for sk=1:length(posStrats(:,1))

                   % pull out the current strategy, and reshape it in a way
                   % that makes it easier to look up the move
                    stratHere=reshape(posStrats(sk,:),2,2,2);

                    % first check if the current strategy is any of the
                    % hardcoded ones; then if not, just look up the move as
                    % specified by the strategy

                    
                    if stratHere(1,1,1)==-9
                        % Grim
                        if been_1_D==1
                            s_imr=-1;
                        else
                            s_imr=1;
                        end
                        
                    elseif stratHere(1,1,1)==-40
                        % Grim-2
                        if G_last2_trig==1
                            s_imr=-1;
                        else
                            s_imr=1;
                        end

                    elseif stratHere(1,1,1)==-41
                        % Grim-3
                        if G_last3_trig==1
                            s_imr=-1;
                        else
                            s_imr=1;
                        end

                    elseif stratHere(1,1,1)==-30
                        % 2PTFT
                        if (you_p_1==1 && other_p_1==1 && you_p_2==1 && other_p_2==1) || ...
                                (you_p_1==1 && other_p_1==1 && you_p_2==2 && other_p_2==2) || ...
                                (you_p_1==2 && other_p_1==2 && you_p_2==1 && other_p_2==1) || ...
                                (you_p_1==2 && other_p_1==2 && you_p_2==2 && other_p_2==2)
                            s_imr=1;
                        else
                            s_imr=-1;
                        end
                    
                    elseif stratHere(1,1,1)==-10
                        % C-to-ALLD
                        if p<=1
                            s_imr=1;
                        else
                            s_imr=-1;
                        end

                    elseif stratHere(1,1,1)==-2.2
                        % 2TF2T
                        if        (other_p_1==2 && other_p_2==2 && other_p_3==1) ...
                                || (other_p_1==1 && other_p_2==2 && other_p_3==2) ...
                                || (other_p_1==2 && other_p_2==2 && other_p_3==2)
                            s_imr=-1;
                        else
                            s_imr=1;
                        end
                        
                    elseif stratHere(1,1,1)==-3
                        % TF3T
                        if other_p_1==2 && other_p_2==2 && other_p_3==2
                            s_imr=-1;
                        else
                            s_imr=1;
                        end
                        
                    elseif stratHere(1,1,1)==-6
                        % D-TFT
                        if p==1
                            s_imr=-1;
                        else
                            if other_p_1==2
                                s_imr=-1;
                            else
                                s_imr=1;
                            end
                        end
                        
                    elseif stratHere(1,1,1)==-7
                        % D-TF2T
                        if p==1
                            s_imr=-1;
                        else
                            if other_p_1==2 && other_p_2==2
                                s_imr=-1;
                            else
                                s_imr=1;
                            end
                        end    
                        
                    elseif stratHere(1,1,1)==-7.1
                        % D-TF3T
                        if p==1
                            s_imr=-1;
                        else
                            if other_p_1==2 && other_p_2==2 && other_p_3==2
                                s_imr=-1;
                            else
                                s_imr=1;
                            end
                        end    

                    elseif stratHere(1,1,1)==-7.15
                        % D-Grim
                        if p==1
                            s_imr=-1;
                        else
                            if been_1_D==1
                                s_imr=-1;
                            else
                                s_imr=1;
                            end
                        end    

                    elseif stratHere(1,1,1)==-7.2
                        % D-Grim2
                        if p==1
                            s_imr=-1;
                        else
                            if G_last2_trig==1
                                s_imr=-1;
                            else
                                s_imr=1;
                            end
                        end    
                        
                    elseif stratHere(1,1,1)==-7.3
                        % D-Grim3
                        if p==1
                            s_imr=-1;
                        else
                            if G_last3_trig==1
                                s_imr=-1;
                            else
                                s_imr=1;
                            end
                        end    
                        
                    elseif stratHere(1,1,1)==-8
                        % DC-Alt
                        s_imr=mod(p,2)*-2+1;

                    elseif stratHere(1,1,1)==-99 
                        % T2 (as per DF 2011)
                        if T2trigger==0
                            s_imr=1;
                        else
                            s_imr=-1;
                        end


                    % otherwise, normal strategy lookup
                    else
                        s_imr=stratHere(other_p_1,you_p_1,other_p_2);
                    end

                % update pis based for subject s and strategy sk based on
                % the actual move y_imr and the predicted move s_imr
                pis(s,sk)=pis(s,sk)*...
                    ((1/(1+exp(-1*s_imr/G))) ^y_imr)...
                    *((1/(1+exp(1*s_imr/G))) ^(1-y_imr));


            end %sk (strategy)
        end %p (period)
    end %m (interaction)
    pis(:,end)=error;
    pis(:,end-1)=bc;

end %s (subject)


